unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Panel1: TPanel;
    Panel2: TPanel;
    GroupBox1: TGroupBox;
    Edit1: TEdit;
    Panel3: TPanel;
    Panel4: TPanel;
    Edit2: TEdit;
    Button2: TButton;
    Button3: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Edit1KeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

// DLL Declarations:
Procedure PSensorGetOffsets(Handle: THandle; var P: Double; var T: Double); stdcall; external 'PSensor.dll';
Procedure PSensorSetOffsets(Handle: THandle; P, T: Double); stdcall; external 'PSensor.dll';
Function PSensorGetHandle: THandle; stdcall; external 'PSensor.dll';
Procedure GetPressureAndTemperature(Handle: THandle; Precision: Byte; var Press: Double; var Temp: Double); stdcall; external 'PSensor.dll';
Procedure PSensorClose(Handle: THandle); stdcall; external 'PSensor.dll';

var
  Form1: TForm1;
  SensorHandle: THandle;

implementation

{$R *.dfm}

// ----------------------------------------------------------------------------
// Get handle of pressure sensor
procedure TForm1.FormCreate(Sender: TObject);
begin
  DecimalSeparator := '.';
  SensorHandle := PSensorGetHandle;                   // Get Handle of sensor
end;

// ----------------------------------------------------------------------------
// Free handle of pressure sensor
procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  PSensorClose(SensorHandle);                         // Close sensor Handle
end;

// ----------------------------------------------------------------------------
// Read pressure and temperature
procedure TForm1.Button1Click(Sender: TObject);
var P, T: Double;
begin
  If SensorHandle <> INVALID_HANDLE_VALUE then        // Handle is valid
  begin
    GetPressureAndTemperature(SensorHandle, 2, P, T); // Get pressure and temp.
    Panel1.Caption := 'Pressure= '+ FloatToStrF(P, ffFixed, 5, 1) + ' hPa.';
    Panel2.Caption := 'Temperature=  '+FloatToStrF(T, ffFixed, 3, 1) + ' C.';
  end
  else                                                // No valid handle
    MessageBox(Handle, 'Sensor not connected?', 'No handle!', MB_OK + MB_IconStop);
end;

// ----------------------------------------------------------------------------
// Read pressure and temperature offset from EEPROM
procedure TForm1.Button2Click(Sender: TObject);
var P, T: Double;
begin
  If SensorHandle <> INVALID_HANDLE_VALUE then
  begin
    PSensorGetOffsets(SensorHandle, P, T);
    Panel3.Caption := FloatToStrF(P, ffFixed, 4, 3);
    Panel4.Caption := FloatToStrF(T, ffFixed, 4, 3);
  end;
end;

// ----------------------------------------------------------------------------
// Write pressure and temperature offset to EEPROM
procedure TForm1.Button3Click(Sender: TObject);
begin
  try
    PSensorSetOffsets(SensorHandle,
                      StrToFloat(Trim(Edit1.Text)),
                      StrToFloat(Trim(Edit2.Text)));
  except
    MessageBox(Handle, 'Calibration of sensor failed', 'Error',
                        MB_OK+MB_IconStop);
  end;
end;

// ----------------------------------------------------------------------------
procedure TForm1.Edit1KeyPress(Sender: TObject; var Key: Char);
begin
  If Key = ',' then Key := '.';                         // Change ',' to '.'
  If ((Key < '0') or (Key > '9')) and (Key <> #8) and   // Only numeric input
     (Key <> '-') and (Key <> '.') then Key:=#0;
end;

end.
