library PSensor;

uses Windows,
     HidUnit in 'HidUnit.pas';

{$R *.res}
{$R VERSION.RES}

var
  Buf: Array[0..8] of Byte;                           // Read buffer
  AC1, AC2, AC3: SmallInt;                            // Cal. variables
  AC4, AC5, AC6: Word;                                // Cal. variables
  B1, B2, B5, MB, MC, MD: SmallInt;                   // Cal. variables
  POffset, TOffset: Double;                           // Cal. Offsets

  bReadCal_1 : array[0..8] of Byte = (0, 1, $82, $77, $AA, 0, 0, 0, 0); // 8 Bytes
  bReadCal_2 : array[0..8] of Byte = (0, 1, $82, $77, $B2, 0, 0, 0, 0); // 8 Bytes
  bReadCal_3 : array[0..8] of Byte = (0, 1, $82, $77, $BA, 0, 0, 0, 0); // 8 Bytes
  bReadPress : array[0..8] of Byte = (0, 1, $84, $66,  0,  0, 0, 0, 0); // 8 Bytes
  bReadTemp  : array[0..8] of Byte = (0, 1, $80, $33,  0,  0, 0, 0, 0); // 8 Bytes
  bReadFlash : array[0..8] of Byte = (0, 1, $83, $77,  1,  0, 0, 0, 0); // 8 Bytes
  bWriteFlash: array[0..8] of Byte = (0, 1, $85, $11,  0,  0, 0, 0, 0); // 8 Bytes

// ****************************************************************************
procedure Delay(mS : DWORD);                          // mSec. delay routine
var t1 : DWORD;
    ProcMsg: TMsg;
begin
  t1 := GetTickCount;
  while (GetTickCount - t1) < mS do
  begin                                               // ProcessMessages
    while PeekMessage(ProcMsg, 0, 0, 0, PM_REMOVE) do
    begin
      if (ProcMsg.Message = $0012) then Exit;
      TranslateMessage(ProcMsg);
      DispatchMessage(ProcMsg);
    end;
  end;
end;

// ****************************************************************************
function GetDeviceString: String;                     // Get MS-DOS devicename
var NumberOfDevices: DWORD;
    i: Integer;
begin
  FindKnownHidDevices(NumberOfDevices);
  for i := 1 to NumberOfDevices do
  begin
    try
      if (GetVendorID(HidDeviceName[i])  = $0C45) or
         (GetProductID(HidDeviceName[i]) = $7405) then
      begin
        Result := HidDeviceName[i];
        Break;
      end;
    except
      //
    end;
  end;
end;

// ****************************************************************************
Procedure PSensorGetOffsets(Handle: THandle; var P: Double; var T: Double); stdcall;
var NumBytes: DWord;
begin
  If Handle <> INVALID_HANDLE_VALUE then
  begin
    WriteFile(Handle, bReadFlash, SizeOf(bReadFlash), NumBytes, nil);
    Delay(2);
    ReadFile(Handle, Buf, SizeOf(Buf), NumBytes, nil);
    If Buf[5] <= $80 Then
      P := Buf[5] * 0.0625
    else
      P := -($100 - Buf[5]) * 0.0625;
    If Buf[6] <= $80 Then
      T := Buf[6] * 0.0625
    else
      T := -($100 -Buf[6]) * 0.0625;
    POffset := P;
    TOffset := T;
  end;
end;

// ****************************************************************************
procedure PSensorSetOffsets(Handle: THandle; P, T: Double); stdcall;
var NumBytes: DWORD;
begin
  If Handle <> INVALID_HANDLE_VALUE then
  begin
    If (P < -7.9) or (P > 8) or
       (T < -7.9) or (T > 8) then
    begin
      MessageBox(0, 'Value must be between -7.9 and +8', 'Error',
                 MB_OK + MB_IconStop);
      Exit;
    end;
    If (P < 0) Then
      bWriteFlash[5]:= $100 - Round(Abs(P * 16))
    else
      bWriteFlash[5] := Round(P * 16);
    If (T < 0) Then
      bWriteFlash[6]:= $100 - Round(Abs(T * 16))
    else
      bWriteFlash[6] := Round(T * 16);
    WriteFile(Handle, bWriteFlash, SizeOf(bWriteFlash), NumBytes, nil);
    Delay(2);
    ReadFile(Handle, Buf, SizeOf(Buf), NumBytes, nil);
    If bWriteFlash[5] <= $80 Then
      POffset := bWriteFlash[5] * 0.0625
    else
      POffset := -($100 - bWriteFlash[5]) * 0.0625;
    If bWriteFlash[6] <= $80 Then
      TOffset := bWriteFlash[6] * 0.0625
    else
      TOffset := -($100 - bWriteFlash[6]) * 0.0625;
  end
  else
    MessageBox(0, 'Can not set offset', 'No device', MB_OK+MB_IconStop);
end;

// ****************************************************************************
procedure GetCalibration(Handle: THandle);
var NumBytes: DWord;
    P, T: Double;
begin
  WriteFile(Handle, bReadCal_1, SizeOf(  bReadCal_1), NumBytes, nil);
  Delay(4);
  ReadFile(Handle, Buf, SizeOf(Buf), NumBytes, nil);
  AC1 := Buf[1] * $100 + Buf[2];
  AC2 := Buf[3] * $100 + Buf[4];
  AC3 := Buf[5] * $100 + Buf[6];
  AC4 := Buf[7] * $100 + Buf[8];
  WriteFile(Handle, bReadCal_2, SizeOf(  bReadCal_2), NumBytes, nil);
  Delay(4);
  ReadFile(Handle, Buf, SizeOf(Buf), NumBytes, nil);
  AC5 := Buf[1] * $100 + Buf[2];
  AC6 := Buf[3] * $100 + Buf[4];
  B1  := Buf[5] * $100 + Buf[6];
  B2  := Buf[7] * $100 + Buf[8];
  WriteFile(Handle, bReadCal_3, SizeOf(  bReadCal_3), NumBytes, nil);
  Delay(4);
  ReadFile(Handle, Buf, SizeOf(Buf), NumBytes, nil);
  MB  := Buf[1] * $100 + Buf[2];
  MC  := Buf[3] * $100 + Buf[4];
  MD  := Buf[5] * $100 + Buf[6];
  PSensorGetOffsets(Handle, P, T);
end;

// ****************************************************************************
Function PSensorGetHandle: THandle; stdcall;          // Get handle for device
var DevString: String;
begin
  Result := INVALID_HANDLE_VALUE;
  DevString := GetDeviceString;
  If DevString <> '' then                             // Device found
  begin
    Result := CreateFile( PChar(DevString), GENERIC_READ or GENERIC_WRITE,
              FILE_SHARE_READ or FILE_SHARE_WRITE, Nil, OPEN_EXISTING, 0, 0);
    If Result <> INVALID_HANDLE_VALUE then GetCalibration(Result);
  end;
end;

// ****************************************************************************
Procedure GetPressureAndTemperature(Handle: THandle; Precision: Byte; var Press: Double; var Temp: Double); stdcall;
var B3, B4, B6, B7: Integer;
    X1, X2, X3: Integer;
    NumBytes: DWord;
    ut, up, p : Integer;
begin
  if Handle <> INVALID_HANDLE_VALUE then
  begin
    WriteFile(Handle, bReadTemp, SizeOf(bReadTemp), NumBytes, nil);
    Delay(2);
    ReadFile(Handle, Buf, SizeOf(Buf), NumBytes, nil);
    ut := Buf[1] * $100 + Buf[2];
    bReadPress[4]:= Precision;
    WriteFile(Handle, bReadPress, SizeOf(bReadPress), NumBytes, nil);
    Case Precision  of
      0: Delay(6);
      1: Delay(11);
      2: Delay(16);
      3: Delay(27);
    end;
    ReadFile(Handle, Buf, SizeOf(Buf), NumBytes, nil);
    up := (Buf[1] * $10000 + Buf[2] * $100 + Buf[3]) Shr (8-Precision);
    X1 := ((ut - AC6) * AC5) div $8000;
    X2 := (MC * $800) div (X1 + MD);
    B5 := X1 + X2;
    B6 := B5 - 4000;
    X1 := (B2 * (B6 * B6 div $1000)) div $800;
    X2 := (AC2 * B6) div $800;
    X3 := X1 + X2;
    B3 := ((AC1 * 4 + X3) SHL Precision + 2) div 4;
    X1 := (AC3 * B6) div $2000;
    X2 := (B1 * ((B6 * B6) div $1000)) div $10000;
    X3 := ((X1 + X2) + 2) div 4;
    B4 := AC4 * (X3 + 32768) div $8000;
    B7 := (up - B3) * (50000 SHR Precision);
    If DWord(B7) < $80000000 then
      p := (B7 div B4) * 2
    else
      P := (B7 * 2) div B4;
    X1 := (p div $100) * (p div $100);
    X1 := (X1 * 3038) div $10000;
    X2 := (-7357 * p) div $10000;
    Temp := TOffset +(B5 + 8) / 160;
    Press:= POffset + (p + ((x1 + x2 + 3791) div $10)) / 100;
  end;
end;

// ----------------------------------------------------------------------------
procedure PSensorClose(Handle: THandle); stdcall;
begin
  If Handle <> INVALID_HANDLE_VALUE then              // Close handle
    CloseHandle(Handle);
end;

// ****************************************************************************
exports
   PSensorGetOffsets,
   PSensorSetOffsets,
   PSensorGetHandle,
   GetPressureAndTemperature,
   PSensorClose;

begin
end.

{ ROUTINES:
Procedure PSensorGetOffsets(Handle: THandle; var P: Double; var T: Double); stdcall; external 'PSensor.dll';
Procedure PSensorSetOffsets(Handle: THandle; P, T: Double); stdcall; external 'PSensor.dll';
Function PSensorGetHandle: THandle; stdcall; external 'PSensor.dll';
Procedure GetPressureAndTemperature(Handle: THandle; Precision: Byte; var Press: Double; var Temp: Double); stdcall; external 'PSensor.dll';
Procedure PSensorClose(Handle: THandle); stdcall; extrenal 'PSensor.dll';
}

