unit HidUnit;

Interface

Uses Windows;

const
  DIGCF_PRESENT              = $00000002;
  DIGCF_DEVICEINTERFACE      = $00000010;

type
  THidD_attributes           = record
    Size                     : DWORD;
    VendorId                 : Word;
    ProductId                : Word;
    VersionNumber            : Word;
  end;
  PHidD_attributes           = ^THidD_attributes;

  HDevInfo                   = Pointer; { a pointer to a HID device info structure }

  TSp_device_interface_data  = record
    cbSize                   : DWORD;
    InterfaceClassGuid       : TGuid;
    Flags                    : DWORD;
    Reserved                 : DWORD;
  end;
  PSp_device_interface_data  = ^TSp_device_interface_data;

  TSp_device_interface_detail_data = packed record
    cbSize                   : DWORD;
    DevicePath               : Array of AnsiChar;
  end;
  PSp_device_interface_detail_data = ^TSp_device_interface_detail_data;

  TSp_devinfo_data           = record
    cbSize                   : DWORD;
    ClassGuid                : TGuid;
    DevInst                  : DWORD;
    Reserved                 : DWORD;
  end;
  PSp_devinfo_data           = ^TSp_devinfo_data;

// --- HID.DLL ---
function  HidD_GetAttributes (HidDeviceObject: THandle;
               Attributes: PHidD_attributes): BOOL; stdcall; external 'HID.DLL';
procedure HidD_GetHidGuid(var TGuid); stdcall; external 'HID.DLL';

// Experimenteel!
function HidD_GetManufacturerString (
          HidDeviceObject: THandle;
          Buffer         :pointer;
          BufferLength   :ULONG
          ): Boolean; stdcall; external 'HID.DLL';

function HidD_GetProductString (
          HidDeviceObject: THandle;
          Buffer         :pointer;
          BufferLength   :ULONG
          ): Boolean; stdcall; external 'HID.DLL';

function HidD_GetSerialNumberString (
          HidDeviceObject: THandle;
          Buffer         :pointer;
          BufferLength   :ULONG
          ):Boolean; stdcall; external 'HID.DLL';

function HidD_GetIndexedString (
          HidDeviceObject: THandle;
          StringIndex    :ULONG;
          Buffer         :pointer;
          BufferLength   :ULONG
          ): Boolean; stdcall external 'HID.DLL';
// Einde experimentele routines

// --- SETUPAPI.DLL ---
function  SetupDiGetDeviceInterfaceDetailA(DeviceInfoSet: HDevInfo;
          DeviceInterfaceData: PSp_device_interface_data;
          DeviceInterfaceDetailData: PSp_device_interface_detail_data;
          DeviceInterfaceDetailDataSize: DWORD; var RequiredSize: DWORD;
          DeviceInfoData: PSp_devinfo_data): BOOL; stdcall; external 'SETUPAPI.DLL';
function  SetupDiGetClassDevsA(var ClassGuid: TGuid; Enumerator: LPCSTR;
          hwndParent: THandle; Flags: DWORD): HDevInfo; stdcall; external 'SETUPAPI.DLL';
function  SetupDiEnumDeviceInterfaces(DeviceInfoSet: HDevInfo;
          DeviceInfoData: PSp_device_interface_data; InterfaceClassGuid: PGuid;
          MemberIndex: DWORD; var DeviceInterfaceData: TSp_device_interface_data
                                      ): BOOL; stdcall; external 'SETUPAPI.DLL';
function SetupDiDestroyDeviceInfoList(DeviceInfoSet: HDevInfo): BOOL;
                                               stdcall; external 'SETUPAPI.DLL';

const TSp_device_interface_detail_data_size = SizeOf( DWORD) + SizeOf( AnsiChar);

var HidDeviceName: Array[ 1..127] of String;

function FindKnownHidDevices( var NumberOfDevices: DWORD): BOOL;
function GetVendorId(DeviceString: String): Word;
function GetProductId(DeviceString: String): Word;

function GetVersionNumber(DeviceString: String): Word;        // 256 (1.0)
function GetManufacturerString(DeviceString: String): String; // 'Van Steenis Software'
function GetProductString(DeviceString: String): String;      // 'Pressure sensor'
function GetSerialNumberString(DeviceString: String): String; // '26012012;

implementation

// ----------------------------------------------------------------------------
function StrLCopy(Dest: PChar; const Source: PChar; MaxLen: Cardinal): PChar; assembler;
asm
        PUSH    EDI
        PUSH    ESI
        PUSH    EBX
        MOV     ESI,EAX
        MOV     EDI,EDX
        MOV     EBX,ECX
        XOR     AL,AL
        TEST    ECX,ECX
        JZ      @@1
        REPNE   SCASB
        JNE     @@1
        INC     ECX
@@1:    SUB     EBX,ECX
        MOV     EDI,ESI
        MOV     ESI,EDX
        MOV     EDX,EDI
        MOV     ECX,EBX
        SHR     ECX,2
        REP     MOVSD
        MOV     ECX,EBX
        AND     ECX,3
        REP     MOVSB
        STOSB
        MOV     EAX,EDX
        POP     EBX
        POP     ESI
        POP     EDI
end;

// ----------------------------------------------------------------------------
function StrPCopy(Dest: PChar; const Source: string): PChar;
begin
  Result := StrLCopy(Dest, PChar(Source), Length(Source));
end;

// ----------------------------------------------------------------------------
function AllocMem(Size: Cardinal): Pointer;
begin
  GetMem(Result, Size);
  FillChar(Result^, Size, 0);
end;

// ----------------------------------------------------------------------------
function GetVendorId(DeviceString: String): Word;
var
  HidDevice: THandle;
  Att: THidD_attributes;
  DN: Array[ 0..255] of Char;
begin
  Result := 0;
  HidDevice := CreateFile(StrPCopy(DN, DeviceString),
                          GENERIC_READ or GENERIC_WRITE,
                          FILE_SHARE_READ or FILE_SHARE_WRITE,
                          Nil, OPEN_EXISTING, 0, 0);
  if HidD_GetAttributes(HidDevice, @Att) then Result := Att.VendorId;
  CloseHandle( HidDevice);
end;

// ----------------------------------------------------------------------------
function GetProductId(DeviceString: String): Word;
var
  HidDevice: THandle;
  Att: THidD_attributes;
  DN: Array[0..255] of Char;
begin
  Result := 0;
  HidDevice := CreateFile(StrPCopy(DN, DeviceString),
                          GENERIC_READ or GENERIC_WRITE,
                          FILE_SHARE_READ or FILE_SHARE_WRITE,
                          Nil, OPEN_EXISTING, 0, 0);
  if HidD_GetAttributes(HidDevice, @Att) then Result := Att.ProductId;
  CloseHandle(HidDevice);
end;

// ----------------------------------------------------------------------------
function FindKnownHidDevices(var NumberOfDevices: DWORD): BOOL;
var
  HardwareDeviceInfo     : HDevInfo;
  DeviceInfoData         : TSp_device_interface_data;
  i                      : Word;
  Done                   : BOOL;
  HidGuid                : TGuid;
  FunctionClassDeviceData: PSp_device_interface_detail_data;
  PredictedLength        : Word;
  RequiredLength         : Word;
  RequiredLength2        : DWORD;
begin
  HidD_GetHidGuid(HidGuid);
  Requiredlength := 0;
  HardwareDeviceInfo := SetupDiGetClassDevsA(HIDGuid, nil, 0,
                          DIGCF_PRESENT or DIGCF_DEVICEINTERFACE);
  DeviceInfoData.cbSize := SizeOf(TSp_device_interface_data);
  i := 0;
  Done := FALSE;
  while not(Done) do
  begin
    if (SetupDiEnumDeviceInterfaces(HardwareDeviceInfo, NIL, @HidGuid, i, DeviceInfoData)) then
    begin
      RequiredLength2 := Requiredlength;
      SetupDiGetDeviceInterfaceDetailA(HardwareDeviceInfo, @DeviceInfoData, NIL, 0, RequiredLength2, NIL);
      Requiredlength := RequiredLength2;
      PredictedLength := RequiredLength;
      FunctionClassDeviceData := AllocMem(PredictedLength);
      FunctionClassDeviceData^.cbSize := TSp_device_interface_detail_data_size;
      RequiredLength2 := Requiredlength;
      if not(SetupDiGetDeviceInterfaceDetailA(HardwareDeviceInfo, @DeviceInfoData,
              FunctionClassDeviceData, PredictedLength, RequiredLength2, NIL))
      then
      begin
         SetupDiDestroyDeviceInfoList(HardwareDeviceInfo);
         FreeMem(FunctionClassDeviceData);
         Result := FALSE;
         Exit;
      end;
      Requiredlength := RequiredLength2;
      HidDeviceName[i + 1] := PChar(@FunctionClassDeviceData^.DevicePath);
      FreeMem(FunctionClassDeviceData);
      i := i + 1;
    end
    else
    begin
      if (GetLastError = ERROR_NO_MORE_ITEMS) then
        Done := TRUE
      else
        Break;
    end;
  end;
  NumberOfDevices := i;
  SetupDiDestroyDeviceInfoList(HardwareDeviceInfo);
  Result := TRUE;
end;

// ----------------------------------------------------------------------------
// Experimenteel
function GetVersionNumber(DeviceString: String): Word;
var
  HidDevice: THandle;
  Att: THidD_attributes;
  DN: Array[ 0..255] of Char;
begin
  Result := 0;
  HidDevice := CreateFile(StrPCopy(DN, DeviceString),
                          GENERIC_READ or GENERIC_WRITE,
                          FILE_SHARE_READ or FILE_SHARE_WRITE,
                          Nil, OPEN_EXISTING, 0, 0);
  if (HidDevice = INVALID_HANDLE_VALUE) then
  begin
    MessageBox(0, 'Could not open device', 'Error', MB_OK+MB_IconStop);
    Exit;
  end;
  if HidD_GetAttributes( HidDevice, @Att) then Result := Att.VersionNumber;
  CloseHandle( HidDevice);
end;

// ----------------------------------------------------------------------------
function GetManufacturerString(DeviceString: String): String;
var
  HidDevice: THandle;
  Str: Array[ 0..255] of WideChar;
  DN: Array[ 0..255] of Char;
begin
  Result := '';
  HidDevice := CreateFile(StrPCopy(DN, DeviceString),
                          GENERIC_READ or GENERIC_WRITE,
                          FILE_SHARE_READ or FILE_SHARE_WRITE,
                          Nil, OPEN_EXISTING, 0, 0);
  if ( HidDevice = INVALID_HANDLE_VALUE) then
  begin
    MessageBox(0, 'Could not open device', 'Error', MB_OK+MB_IconStop);
    Exit;
  end;
  if HidD_GetManufacturerString( HidDevice, @Str, SizeOf( Str)) then
    Result := WideCharToString( Str);
  CloseHandle( HidDevice);
end;

// ----------------------------------------------------------------------------
function GetProductString(DeviceString: String): String;
var
  HidDevice: THandle;
  Str: Array[ 0..255] of WideChar;
  DN: Array[ 0..255] of Char;
begin
  Result := '';
  HidDevice := CreateFile(StrPCopy(DN, DeviceString),
                          GENERIC_READ or GENERIC_WRITE,
                          FILE_SHARE_READ or FILE_SHARE_WRITE,
                          Nil, OPEN_EXISTING, 0, 0);
  if ( HidDevice = INVALID_HANDLE_VALUE) then
  begin
    MessageBox(0, 'Could not open device', 'Error', MB_OK+MB_IconStop);
    Exit;
  end;
  if HidD_GetProductString( HidDevice, @Str, SizeOf( Str)) then
    Result := WideCharToString( Str);
  CloseHandle( HidDevice);
end;

// ----------------------------------------------------------------------------
function GetSerialNumberString(DeviceString: String): String;
var
  HidDevice: THandle;
  Str: Array[ 0..255] of WideChar;
  DN: Array[ 0..255] of Char;
begin
  Result := '';
  HidDevice := CreateFile(StrPCopy(DN, DeviceString),
                           GENERIC_READ or GENERIC_WRITE,
                           FILE_SHARE_READ or FILE_SHARE_WRITE,
                           Nil, OPEN_EXISTING, 0, 0);
  if ( HidDevice = INVALID_HANDLE_VALUE) then
  begin
    MessageBox(0, 'Could not open device', 'Error', MB_OK+MB_IconStop);
    Exit;
  end;
  if HidD_GetSerialNumberString( HidDevice, @Str, SizeOf( Str)) then
    Result := WideCharToString( Str);
  CloseHandle( HidDevice);
end;

// ----------------------------------------------------------------------------
end.


